//****************************************************************************
// 3dtest.c
// (c) 1992 by Technological Computer Innovations
// Author: D. Metzler
//****************************************************************************

#include <windows.h>
#include <string.h>
#include <stdio.h>
#include "3dtest.h"
#include "3d.h"

int PASCAL WinMain(HINSTANCE hInstance, HINSTANCE hPrevInstance, LPSTR lpCmdLine, int nCmdShow)
{
    MSG msg;

    if (!hPrevInstance)
        if (!InitApplication(hInstance))
            return (FALSE) ;

    if (!InitInstance(hInstance, nCmdShow))
        return (FALSE) ;

    while (GetMessage(&msg, NULL, NULL, NULL))
	{
        TranslateMessage(&msg) ;
        DispatchMessage(&msg) ;
    }

    return (msg.wParam) ;
}

BOOL InitApplication(HANDLE hInstance)
{
    WNDCLASS  wc ;

    wc.style            = NULL ;
    wc.lpfnWndProc      = MainWndProc ;
    wc.cbClsExtra       = 0 ;
    wc.cbWndExtra       = 0 ;
    wc.hInstance        = hInstance ;
    wc.hIcon            = LoadIcon(hInstance, "ThreeDIcon") ;
    wc.hCursor          = LoadCursor(NULL, IDC_ARROW) ;
    wc.hbrBackground    = GetStockObject(LTGRAY_BRUSH) ;
    wc.lpszMenuName     = "CustomMenu" ;
    wc.lpszClassName    = "CustomWClass" ;

    return (RegisterClass(&wc)) ;
}

BOOL InitInstance(HANDLE hInstance, int nCmdShow)
{
    HWND            hWnd;

    hInst = hInstance ;

    /* Create a main window for this application instance.  */

    hWnd = CreateWindow(
        "CustomWClass",
        "3D Effects",
        WS_OVERLAPPEDWINDOW,
        CW_USEDEFAULT,
        CW_USEDEFAULT,
        CW_USEDEFAULT,
        CW_USEDEFAULT,
        NULL,
        NULL,
        hInstance,
        NULL) ;

    if (!hWnd)
        return (FALSE) ;

    /* Place window in center of screen */
    CenterWindow(hWnd, CW_SCREEN) ;
    ShowWindow(hWnd, nCmdShow) ;
    UpdateWindow(hWnd) ;

    return (TRUE) ;
}

LONG FAR PASCAL MainWndProc(HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam)
{
    PAINTSTRUCT ps ;

    switch (message)
    {
        case WM_COMMAND:
            switch(wParam)
            {
                case IDM_ABOUT:
                    lpProcAbout = MakeProcInstance((FARPROC) AboutProc, hInst) ;
                    DialogBox(hInst, "AboutDlg", hWnd, (DLGPROC) lpProcAbout) ;
                    FreeProcInstance(lpProcAbout) ;
                    break ;

                case IDM_DLG1:
                    lpProcAbout = MakeProcInstance((FARPROC) SampleProc, hInst) ;
                    DialogBox(hInst, "SampleDlg", hWnd, (DLGPROC) lpProcAbout) ;
                    FreeProcInstance(lpProcAbout) ;
                    break ;

                case IDM_DLG2:
                    lpProcAbout = MakeProcInstance((FARPROC) Sample3DProc, hInst) ;
                    DialogBox(hInst, "SampleDlg", hWnd, (DLGPROC) lpProcAbout) ;
                    FreeProcInstance(lpProcAbout) ;
                    break ;

                case IDM_EXIT:
                    DestroyWindow(hWnd) ;
                    break ;
            }
            return (TRUE) ;

        case WM_SIZE:
            InvalidateRect(hWnd, NULL, TRUE) ;
            return (TRUE) ;

        case WM_PAINT:
            BeginPaint(hWnd, &ps) ;
            EndPaint(hWnd, &ps) ;
            PlaceCornerRivets(hWnd) ;
            return (TRUE) ;

        case WM_DESTROY:
            PostQuitMessage(0) ;
            break ;

        default:
            return (DefWindowProc(hWnd, message, wParam, lParam)) ;
    }
    return (NULL) ;
}

void CenterWindow(HWND hWnd, int type)
{
    int     cx, cy ;
    RECT    rc ;

    switch(type)
    {
        case CW_SCREEN:
            // Center within screen
            cx = (GetSystemMetrics(SM_CXSCREEN) / 2) ;
            cy = (GetSystemMetrics(SM_CYSCREEN) / 2) ;

            GetClientRect(hWnd, (LPRECT) &rc) ;
            cx -= ((rc.right - rc.left) / 2) ;
            cy -= ((rc.bottom - rc.top) / 2) ;
            SetWindowPos(hWnd, NULL, cx, cy, NULL, NULL, SWP_NOSIZE | SWP_NOZORDER) ;
            break ;

        case CW_MAIN:
            // Center window within parent
            GetWindowRect(GetParent(hWnd), &rc) ;
            cx = (rc.right + rc.left) / 2 ;
            cy = (rc.top + rc.bottom) / 2 ;

            GetClientRect(hWnd, (LPRECT) &rc) ;
            cx -= ((rc.right - rc.left) / 2) ;
            cy -= ((rc.bottom - rc.top) / 2) ;
            SetWindowPos(hWnd, NULL, cx, cy, NULL, NULL, SWP_NOSIZE | SWP_NOZORDER) ;
            break ;
    }
}

void BigText(HDC hDC, int x, int y, LPSTR lpstring, int ln)
{
    LOGFONT     lf ;
    HFONT       hFont ;
    HANDLE      hOld ;

    lf.lfHeight = 20 ;
    lf.lfWidth = 0 ;
    lf.lfEscapement = 0 ;
    lf.lfOrientation = 0 ;
    lf.lfWeight = 700 ;
    lf.lfItalic = 0 ;
    lf.lfUnderline = 0 ;
    lf.lfStrikeOut = 0 ;
    lf.lfCharSet = ANSI_CHARSET ;
    lf.lfOutPrecision = OUT_DEFAULT_PRECIS ;
    lf.lfClipPrecision = CLIP_DEFAULT_PRECIS ;
    lf.lfQuality = PROOF_QUALITY ;
    lf.lfPitchAndFamily = VARIABLE_PITCH | FF_DONTCARE ;
    strcpy(lf.lfFaceName, "Helv") ;

    hFont = CreateFontIndirect(&lf) ;
    hOld = SelectObject(hDC, hFont) ;
    TextOut(hDC, x, y, lpstring, ln) ;
    SelectObject(hDC, hOld) ;
    DeleteObject(hFont) ;
}

LONG FAR PASCAL AboutProc(HWND hDlg, UINT message, WPARAM wParam, LPARAM lParam)
{
    PAINTSTRUCT     ps ;

    switch(message)
    {
        case WM_INITDIALOG:
            ShowWindow(GetDlgItem(hDlg, ID_GROUP1), SW_HIDE) ;
            ShowWindow(GetDlgItem(hDlg, ID_FRAME1), SW_HIDE) ;
            ShowWindow(GetDlgItem(hDlg, ID_FRAME2), SW_HIDE) ;
            CenterWindow(hDlg, CW_SCREEN) ;
            return (TRUE) ;

        case WM_CTLCOLOR:
            switch(HIWORD(lParam))
            {
                case CTLCOLOR_DLG:
                case CTLCOLOR_STATIC:
                    SetBkColor((HDC) wParam, LTGRAY) ;
                    SetTextColor((HDC) wParam, BLACK) ;
                    return ((LRESULT)(void far *) GetStockObject(LTGRAY_BRUSH)) ;

                default:
                    return (FALSE) ;
            }
            break ;

        case WM_PAINT:
            BeginPaint(hDlg, &ps) ;
            ShadowArea(hDlg, ps.hdc, IDOK, GetStockObject(DKGRAY_BRUSH)) ;
            Recess(hDlg, ps.hdc, ID_FRAME1, 2) ;
            Raise(hDlg, ps.hdc, ID_FRAME2, 2) ;
            EndPaint(hDlg, &ps) ;
            PlaceCornerRivets(hDlg) ;
            MakeGroup3D(hDlg, ID_GROUP1, 1) ;
            return (TRUE) ;

        case WM_COMMAND:
            switch(wParam)
            {
                case IDOK:
                    EndDialog(hDlg, TRUE) ;
                    return (TRUE) ;
            }
            return (TRUE) ;
    }
    return (FALSE) ;
}

LONG FAR PASCAL SampleProc(HWND hDlg, UINT message, WPARAM wParam, LPARAM lParam)
{
    int i ;
    static int  iScroll ;

    switch(message)
    {
        case WM_INITDIALOG:
            SetWindowText(hDlg, "Sample Vanilla Dialog") ;
            CenterWindow(hDlg, CW_SCREEN) ;
            iScroll = 0 ;
            SetWindowText(GetDlgItem(hDlg, ID_EDIT), "Edit text") ;
            SetDlgItemInt(hDlg, ID_STATIC, iScroll, FALSE) ;
            SetScrollRange(GetDlgItem(hDlg, ID_SCROLL1), SB_CTL, 0, 100, FALSE) ;
            SetScrollPos(GetDlgItem(hDlg, ID_SCROLL1), SB_CTL, iScroll, TRUE) ;
            return (TRUE) ;

        case WM_COMMAND:
            switch(wParam)
            {
                case IDOK:
                case IDCANCEL:
                    EndDialog(hDlg, TRUE) ;
                    return (TRUE) ;

                case ID_FILL:
                    FillListbox(hDlg, ID_LISTBOX1) ;
                    return (TRUE) ;
            }
            return (TRUE) ;

        case WM_HSCROLL:
            i = iScroll ;
            switch(wParam)
            {
                case SB_PAGEDOWN:
                case SB_LINEDOWN:
                    i++ ;
                    break ;

                case SB_PAGEUP:
                case SB_LINEUP:
                    i-- ;
                    break ;

                case SB_TOP:
                    i = 0 ;
                    break ;

                case SB_BOTTOM:
                    i = 100 ;
                    break ;

                case SB_THUMBPOSITION:
                case SB_THUMBTRACK:
                    i = (int) LOWORD(lParam) ;
                    break ;

                default:
                    return (FALSE) ;
            }

            if (i < 0)
                i = 0 ;
            if (i > 100)
                i = 100 ;

            if (i != iScroll)
            {
                iScroll = i ;
                SetDlgItemInt(hDlg, ID_STATIC, iScroll, FALSE) ;
                SetScrollPos(GetDlgItem(hDlg, ID_SCROLL1), SB_CTL, iScroll, TRUE) ;
            }
            return (TRUE) ;

    }
    return (FALSE) ;
}

LONG FAR PASCAL Sample3DProc(HWND hDlg, UINT message, WPARAM wParam, LPARAM lParam)
{
    PAINTSTRUCT     ps ;
    int             i ;
    static int      iScroll ;

    switch(message)
    {
        case WM_INITDIALOG:
            SetWindowText(hDlg, "Sample 3D Dialog") ;
            ShowWindow(GetDlgItem(hDlg, ID_FRAME1), SW_HIDE) ;
            ShowWindow(GetDlgItem(hDlg, ID_FRAME2), SW_HIDE) ;
            ShowWindow(GetDlgItem(hDlg, ID_GROUP1), SW_HIDE) ;
            ShowWindow(GetDlgItem(hDlg, ID_GROUP2), SW_HIDE) ;
            iScroll = 0 ;
            SetDlgItemInt(hDlg, ID_STATIC, iScroll, FALSE) ;
            SetScrollRange(GetDlgItem(hDlg, ID_SCROLL1), SB_CTL, 0, 100, FALSE) ;
            SetScrollPos(GetDlgItem(hDlg, ID_SCROLL1), SB_CTL, iScroll, TRUE) ;
            CenterWindow(hDlg, CW_SCREEN) ;
            SetWindowText(GetDlgItem(hDlg, ID_EDIT), "Edit text") ;
            return (TRUE) ;

        case WM_CTLCOLOR:
            switch(HIWORD(lParam))
            {
                case CTLCOLOR_DLG:
                    return ((LRESULT)(void far *) GetStockObject(LTGRAY_BRUSH)) ;

                default:
                    return (FALSE) ;
            }
            break ;

        case WM_PAINT:
            BeginPaint(hDlg, &ps) ;
            ShadowArea(hDlg, ps.hdc, IDOK, GetStockObject(DKGRAY_BRUSH)) ;
            ShadowArea(hDlg, ps.hdc, ID_LISTBOX1, GetStockObject(DKGRAY_BRUSH)) ;
            ShadowArea(hDlg, ps.hdc, ID_SCROLL1, GetStockObject(DKGRAY_BRUSH)) ;
            Recess(hDlg, ps.hdc, ID_FRAME1, 2) ;
            Recess(hDlg, ps.hdc, ID_STATIC, 2) ;
            Recess(hDlg, ps.hdc, ID_EDIT, 2) ;
            Raise(hDlg, ps.hdc, ID_FRAME2, 2) ;
            Place3DText(ps.hdc, 233, 150, "3D Text") ;
            EndPaint(hDlg, &ps) ;
            PlaceCornerRivets(hDlg) ;
            MakeGroup3D(hDlg, ID_GROUP1, 1) ;
            MakeGroup3D(hDlg, ID_GROUP2, 0) ;
            return (TRUE) ;

        case WM_HSCROLL:
            i = iScroll ;
            switch(wParam)
            {
                case SB_PAGEDOWN:
                case SB_LINEDOWN:
                    i++ ;
                    break ;

                case SB_PAGEUP:
                case SB_LINEUP:
                    i-- ;
                    break ;

                case SB_TOP:
                    i = 0 ;
                    break ;

                case SB_BOTTOM:
                    i = 100 ;
                    break ;

                case SB_THUMBPOSITION:
                case SB_THUMBTRACK:
                    i = (int) LOWORD(lParam) ;
                    break ;

                default:
                    return (FALSE) ;
            }

            if (i < 0)
                i = 0 ;
            if (i > 100)
                i = 100 ;

            if (i != iScroll)
            {
                iScroll = i ;
                SetDlgItemInt(hDlg, ID_STATIC, iScroll, FALSE) ;
                SetScrollPos(GetDlgItem(hDlg, ID_SCROLL1), SB_CTL, iScroll, TRUE) ;
            }
            return (TRUE) ;

        case WM_COMMAND:
            switch(wParam)
            {
                case IDOK:
                case IDCANCEL:
                    EndDialog(hDlg, TRUE) ;
                    return (TRUE) ;

                case ID_FILL:
                    FillListbox(hDlg, ID_LISTBOX1) ;
                    return (TRUE) ;
            }
            return (TRUE) ;
    }
    return (FALSE) ;
}

void FillListbox(HWND hDlg, int id)
{
    char    buffer[32] ;
    int     i ;

    SendMessage(GetDlgItem(hDlg, id), LB_RESETCONTENT, NULL, NULL) ;
    for(i=1;i<=25;i++)
    {
        sprintf(buffer, "Item #%d", i) ;
        SendMessage(GetDlgItem(hDlg, id), LB_ADDSTRING, NULL, (LONG)(LPSTR) buffer) ;
    }
}
