/*
 *    ccdlg.c - Prototype dialog editor interface for a custom control.
 *              This code is designed to supply a complete interface to
 *              both Microsoft's Dialog Editor and Borland's Resource
 *              Workshop.
 */

#ifdef __BORLANDC__
    #undef STRICT   /* work around Borland header file bugs */
    #undef WINVER
#endif

#include <windows.h>
#include <custcntl.h>
#include <string.h>
#include <memory.h>

#define TACKY_TRICK
#include "ccdlg.h" 

HINSTANCE    ModuleInstance;

/* CTLTYPE_EX - extended version for Resource Workshop */

typedef struct  CTLTYPE_EX
    {
    CTLTYPE     CtlType;
    HBITMAP     ToolboxBitmap;
    HCURSOR     DndCursor;
    }           CTLTYPE_EX;

/* CTLSTYLE_EX  - extended version for Resource Workshop */

typedef struct  CTLSTYLE_EX
    {
    CTLSTYLE    CtlStyle;
    BYTE        CtlDataSize;
    BYTE        CtlData[255];
    };

int FAR PASCAL _WEP(int ExitCode)
    {
    extern  int CC_UNREGISTER_FUNC(void);

    switch(ExitCode)
        {
        case    WEP_SYSTEM_EXIT :
        case    WEP_FREE_DLL    :
            CC_UNREGISTER_FUNC();
            return 1;
        }
    return 1;
    }


#ifdef __BORLANDC__
    #pragma argsused
#endif
int  CALLBACK LibMain(HINSTANCE Module, WORD DataSegment,
                    WORD HeapSize, LPSTR CommandLine)
    {
    extern  int CC_REGISTER_FUNC(HINSTANCE);

    if(HeapSize != 0)
        UnlockData(0);

    ModuleInstance    = Module;
    if(CC_REGISTER_FUNC(Module))
        return 1;
    else
        {
        MessageBox(NULL, "Could not register custom control "
                CC_CLASS_NAME, "", MB_OK);
        return 0;
        }
    }

CTLTYPE    CtlType    =
    {
    0, CC_DEFAULT_WIDTH, CC_DEFAULT_HEIGHT,
        CC_DEFAULT_STYLE,
    { CC_CLASS_NAME }
    };

static
HGLOBAL     OldStyleInfo(int ExtraBytes)
    {
    HGLOBAL    ReturnHandle;
    CTLINFO FAR *Info;
    size_t      InfoSize;

    InfoSize        = sizeof(CTLINFO) - (CTLTYPES-1)*sizeof(CTLTYPE);
    InfoSize       += ExtraBytes;
    ReturnHandle    = GlobalAlloc(GHND, InfoSize);
    if(ReturnHandle)
        {
        Info    = (CTLINFO FAR *)GlobalLock(ReturnHandle);
        if(!Info)
            {
            GlobalFree(ReturnHandle);
            return FALSE;
            }
        Info->wVersion  = CC_VERSION;
        Info->wCtlTypes = 1;    /* only one control sub-type */
        _fstrcpy(&Info->szClass[0], CC_CLASS_NAME);
        _fstrcpy(&Info->szTitle[0], CC_TITLE);
        _fmemcpy(&Info->Type[0], &CtlType, sizeof(CtlType));
        GlobalUnlock(ReturnHandle);    
        }
    
    return ReturnHandle;
    }


HGLOBAL    WINAPI _export CC_INFO_FUNC(void)
    {
    return OldStyleInfo(0);
    }


UINT FAR PASCAL _export CC_FLAGS_FUNC(DWORD Flags,
                        LPSTR Buffer, UINT MaxString)
    {
    int     i;

    for(i = 0; i < sizeof(CcFlags)/sizeof(CC_FLAG); ++i)
        if(Flags & CcFlags[i].Flag)
            if(_fstrlen(Buffer) + strlen(CcFlags[i].Symbol) + 3 >= MaxString)
                return 0;   /* failed -- string too long */
            else
                {
                if(*Buffer)
                    _fstrcat(Buffer, "|");
                _fstrcat(Buffer, CcFlags[i].Symbol);
                }
    return _fstrlen(Buffer);
    }

BOOL FAR PASCAL _export CC_STYLE_FUNC(HWND Parent, GLOBALHANDLE StyleData,
                LPFNSTRTOID StrToId, LPFNIDTOSTR IdToStr)
    {
    extern  BOOL FAR PASCAL _export
        CC_DIALOG_FUNC(HWND Dialog, UINT Message,
                        WPARAM WordParm, LPARAM LongParm);
    CC_INIT_DIALOG  InitInfo;

    InitInfo.Extended   = FALSE;
    InitInfo.StyleData  = StyleData;
    InitInfo.StrToId    = StrToId;
    InitInfo.IdToStr    = IdToStr;

    return DialogBoxParam(ModuleInstance, CC_DIALOG_NAME,
        Parent, CC_DIALOG_FUNC, (LPARAM)((CC_INIT_DIALOG FAR *)&InitInfo));
    }


/*********** The Resource Workshop extended interface ********/

/* Avoid requiring inclusion of Borland's custcntl.h */

#ifndef __BORLANDC__

typedef HGLOBAL (CALLBACK *LPFNLOADRES)(LPCSTR, LPCSTR);
typedef BOOL    (CALLBACK *LPFNEDITRES)(LPCSTR, LPCSTR);
typedef HGLOBAL (CALLBACK *LPFNINFO)(void);
typedef BOOL (CALLBACK *LPFNSTYLE)(HWND, HGLOBAL, LPFNSTRTOID, LPFNIDTOSTR);
typedef UINT (CALLBACK *LPFNFLAGS)(DWORD, LPSTR, UINT);
typedef struct  RWCTLCLASS
    {
    LPFNINFO    fnRWInfo;
    LPFNSTYLE   fnRWStyle;
    LPFNFLAGS   fnFlags;
    char        szClass[CTLCLASS];
    }           RWCTLCLASS;

typedef struct CTLCLASSLIST
    {
    short       nClasses;
    RWCTLCLASS  Classes[1];
    }          CTLCLASSLIST;

typedef struct  RWCTLTYPE
    {
    UINT        wType;
    UINT        wWidth;
    UINT        wHeight;
    DWORD       dwStyle;
    char        szDescr[CTLDESCR];
    HBITMAP     hToolBit;
    HCURSOR     hDropCurs;
    }           RWCTLTYPE;

typedef struct  RWCTLINFO
    {
    UINT        wVersion;
    }           RWCTLINFO;

#endif

HGLOBAL    WINAPI _export RwInfo(void)
    {
    HGLOBAL Handle;
    Handle  = OldStyleInfo(sizeof(RWCTLTYPE) - sizeof(CTLTYPE));
    if(Handle)
        {
        HBITMAP FAR *ToolboxBitmap;
        HCURSOR FAR *DndCursor;
        CTLINFO FAR *Info;
        int         InfoSize;
        Info    = (CTLINFO FAR *)GlobalLock(Handle);
        if(Info)
            {
            InfoSize        = sizeof(CTLINFO) - (CTLTYPES-1)*sizeof(CTLTYPE);
            ToolboxBitmap   = (HBITMAP FAR *)((char FAR *)Info + InfoSize);
            DndCursor       = (HCURSOR FAR *)((char FAR *)ToolboxBitmap + sizeof(HBITMAP));
            *ToolboxBitmap  = LoadBitmap(ModuleInstance, CC_BITMAP_NAME);
            *DndCursor      = LoadCursor(ModuleInstance, CC_CURSOR_NAME);
            }
        }
    return Handle;
    }

BOOL FAR PASCAL _export RwStyle(HWND Parent, GLOBALHANDLE StyleData,
                LPFNSTRTOID StrToId, LPFNIDTOSTR IdToStr)
    {
    extern  BOOL FAR PASCAL _export
        CC_DIALOG_FUNC(HWND Dialog, UINT Message,
                        WPARAM WordParm, LPARAM LongParm);
    CC_INIT_DIALOG  InitInfo;

    InitInfo.Extended   = TRUE;
    InitInfo.StyleData  = StyleData;
    InitInfo.StrToId    = StrToId;
    InitInfo.IdToStr    = IdToStr;
    return DialogBoxParam(ModuleInstance, CC_DIALOG_NAME,
        Parent, CC_DIALOG_FUNC, (LPARAM)(CC_INIT_DIALOG FAR *)&InitInfo);
    }


#ifdef __BORLANDC__
    #pragma argsused
#endif
HGLOBAL CALLBACK _export ListClasses(LPSTR DlgEdClass, UINT DlgEdVersion,
            LPFNLOADRES LoadResource, LPFNEDITRES EditResource)
    {
    HGLOBAL    ReturnHandle;
    CTLCLASSLIST    FAR *Memory;

    ReturnHandle    = GlobalAlloc(GHND, sizeof(CTLCLASSLIST));
    if(ReturnHandle)
        {
        Memory  = (CTLCLASSLIST FAR *)GlobalLock(ReturnHandle);
        if(!Memory)
            {
            GlobalFree(ReturnHandle);
            return FALSE;
            }
        Memory->nClasses    = 1;
        Memory->Classes[0].fnRWInfo     = RwInfo;
	Memory->Classes[0].fnRWStyle	= RwStyle;
        Memory->Classes[0].fnFlags      = CC_FLAGS_FUNC;
        _fstrcpy(Memory->Classes[0].szClass, CC_CLASS_NAME);
        GlobalUnlock(ReturnHandle);    
        }
    return ReturnHandle;
    }
