/*****************************************************/
/* mdi.c                                             */
/* -- Demo MDI app uses a child edit control.        */
/*****************************************************/

#include <windows.h>
#include <windowsx.h>
#include "mdi.h"

/* Globals. */
HWND        hwndClient;     /* MDI client. */
HWND        hwndActive;     /* Active MDI child. */
HWND        hwndEdit;       /* Local edit window. */
WNDPROC     lpfnEdit;       /* Edit window proc. */

/* Class names. */
char    szFrame[]       = "frame";   /* Frame. */
char    szChild[]       = "child";   /* MDI child. */
char    szLocalEdit[]   = "ledit";   /* Local edit. */

LRESULT CALLBACK ChildWndProc(HWND, UINT, WPARAM,
                                 LPARAM);
LRESULT CALLBACK FrameWndProc(HWND, UINT, WPARAM,
                              LPARAM);
LRESULT CALLBACK LocalEditWndProc(HWND, UINT, WPARAM,
                                  LPARAM);

int PASCAL
WinMain(HINSTANCE hinsThis, HINSTANCE hinsPrev,
  LPSTR lszCmdLine, int wCmdShow)
/*****************************************************/
/* -- Entry point.                                   */
/*****************************************************/
    {
    MSG     msg;
    HWND    hwnd;
    RECT    rect;

    /* If this is the first instance of the app. */
    /* register window classes. */
    if (!hinsPrev)
        {
        WNDCLASS  wcs;

        /* Register the frame class. */
        wcs.style = 0;
        wcs.lpfnWndProc = FrameWndProc;
        wcs.cbClsExtra = 0;
        wcs.cbWndExtra = 0;
        wcs.hInstance = hinsThis;
        wcs.hIcon = LoadIcon(NULL, IDI_APPLICATION);
        wcs.hCursor = LoadCursor(NULL, IDC_ARROW);
        wcs.hbrBackground =
          (HBRUSH)(COLOR_APPWORKSPACE + 1);
        wcs.lpszMenuName = MAKEINTRESOURCE(idmMDI);
        wcs.lpszClassName = szFrame;
        if (!RegisterClass(&wcs))
            return FALSE;

        /* Register the MDI child class. */
        wcs.lpfnWndProc = ChildWndProc;
        wcs.hIcon = LoadIcon(NULL, IDI_APPLICATION);
        wcs.lpszMenuName = NULL;
        wcs.cbWndExtra = 0;
        wcs.lpszClassName = szChild;
        if (!RegisterClass(&wcs))
            return FALSE;

        /* Register the local edit class. */
        GetClassInfo(NULL, "edit", &wcs);
        lpfnEdit = wcs.lpfnWndProc;
        wcs.hInstance = hinsThis;
        wcs.lpfnWndProc = LocalEditWndProc;
        wcs.lpszClassName = szLocalEdit;
        if (!RegisterClass(&wcs))
            return FALSE;
        }

    /* Create the frame. */
    if (!(hwnd = CreateWindow(szFrame, "MDI",
      WS_OVERLAPPEDWINDOW,
      CW_USEDEFAULT, 0, CW_USEDEFAULT, 0,
      NULL, NULL, hinsThis, NULL)))
        return FALSE;

    /* Create the edit control at top left. */
    GetClientRect(hwnd, &rect);
    hwndEdit = CreateWindow(szLocalEdit, NULL,
      WS_BORDER | WS_CHILD | ES_MULTILINE |
        WS_VISIBLE | WS_CLIPSIBLINGS,
      0, 0, rect.right / 2, rect.bottom / 2,
      hwndClient, (HMENU)1, hinsThis, NULL);

    ShowWindow(hwnd, wCmdShow);     /* Show frame. */
    while (GetMessage(&msg, NULL, 0, 0))
        if (!TranslateMDISysAccel(hwndClient, &msg))
            {
            TranslateMessage(&msg);
            DispatchMessage(&msg);
            }

    return msg.message;
    }

LRESULT CALLBACK
FrameWndProc(HWND hwnd, UINT wm, WPARAM wParam,
  LPARAM lParam)
/*****************************************************/
/* -- Frame window procedure.                        */
/*****************************************************/
    {
    switch (wm)
        {
    default:
        return DefFrameProc(hwnd, hwndClient, wm,
          wParam, lParam);

    case WM_CREATE:
        {
        CLIENTCREATESTRUCT  ccs;

        /* Find window menu where children will be */
        /* listed. */
        ccs.hWindowMenu = GetSubMenu(GetMenu(hwnd), 1);
        ccs.idFirstChild = 0x1000;

        /* Create the MDI client. */
        hwndClient = CreateWindow("mdiclient", NULL,
          WS_CHILD | WS_VISIBLE, 0, 0, 0, 0, hwnd, 0,
          GetWindowInstance(hwnd), (LPSTR)&ccs);
        }
        break;

    case WM_COMMAND:
        switch (wParam)
            {
        default:
            DefFrameProc(hwnd, hwndClient, wm, wParam,
              0L);
            break;

        case idmFileNew:
            {
            /* Make an empty MDI child window. */
            MDICREATESTRUCT mcs;

            mcs.szTitle = "MDI Child";
            mcs.szClass = szChild;
            mcs.hOwner = GetWindowInstance(hwnd);
            mcs.x = mcs.cx = mcs.y = mcs.cy =
              CW_USEDEFAULT;
            mcs.style = 0;

            /* Tell the MDI Client to create the */
            /* child. */
            if (SendMessage(hwndClient, WM_MDICREATE,
              0, (LONG)(LPMDICREATESTRUCT)&mcs))
                ShowWindow(hwnd, SW_SHOW);
            break;
            }
        break;

    case WM_CLOSE:
        DestroyWindow(hwnd);
        break;

    case WM_DESTROY:
        PostQuitMessage(0);
        break;

    case WM_SETFOCUS:
        if (hwndEdit)
            SetFocus(hwndEdit);
        break;
        }
    return 0;
    }

LRESULT CALLBACK
ChildWndProc(HWND hwnd, UINT wm, WPARAM wParam,
  LPARAM lParam)
/*****************************************************/
/* -- MDI child window procedure.                    */
/*****************************************************/
    {
    switch (wm)
        {
    default:
        break;

    case WM_MDIACTIVATE:
        if (wParam)
            hwndActive = hwnd;
        break;

    case WM_PAINT:
        {
        PAINTSTRUCT  wps;
        HLOCAL       hmem;

        BeginPaint(hwnd, &wps);
        if (hmem = (HLOCAL)SendMessage(hwndEdit,
          EM_GETHANDLE, 0, 0))
            {
            RECT    rect;
            PSTR    sz   = LocalLock(hmem);

            GetClientRect(hwnd, &rect);
            DrawText(wps.hdc, sz, -1, &rect,
              DT_NOPREFIX | DT_WORDBREAK);
            LocalUnlock(hmem);
            }
        EndPaint(hwnd, &wps);
        }
        return 0;
        }

    return DefMDIChildProc(hwnd, wm, wParam, lParam);
    }

LRESULT CALLBACK
LocalEditWndProc(HWND hwnd, UINT wm, WPARAM wParam,
  LPARAM lParam)
/*****************************************************/
/* -- Subclasser for edit window.                    */
/* -- Copy text of edit to active MDI child.         */
/*****************************************************/
    {
    LRESULT lval;

    lval = CallWindowProc(lpfnEdit, hwnd, wm, wParam,
      lParam);

    switch (wm)
        {
    default:
        break;

    case WM_KEYUP:
        InvalidateRect(hwndActive, NULL, TRUE);
        break;

    case WM_SETFOCUS:
        BringWindowToTop(hwnd);
        break;
        }

    return lval;
    }
