/*****************************************************/
/* syskey.c                                          */
/* -- Dll for use by Microsoft Setup Utility to      */
/*    place a key-value pair in the system.ini file  */
/*    the avoid duplicate key-value pairs.           */
/*****************************************************/

#include <windows.h>
#include <windowsx.h>
#include <limits.h>

/* Private. */
BOOL                 FIsWhite(int);
LPSTR                LpchSkipWhite(LPSTR);
int                  WReadLineLsz(int, LPSTR, int);

/* Exports. */
int CALLBACK _export LibMain(HINSTANCE, WORD, WORD,
                             LPSTR);
int CALLBACK _export WEP(int);

#define cbKeyValueMax   1024    /* Max. .ini line. */

int CALLBACK _export
LibMain(HINSTANCE hinsThis, WORD sel, WORD cbHeap,
  LPSTR lsz)
/*****************************************************/
/* -- Entry point.                                   */
/*****************************************************/
    {
    return TRUE;
    }

int CALLBACK _export
WEP(int nExitCode)
/*****************************************************/
/* -- Exit proc.                                     */
/*****************************************************/
    {
    return 1;
    }

int PASCAL _export
FExistsSysIniKeyValue(LPSTR lszFile, LPSTR lszSection,
  LPSTR lszKey, LPSTR lszValue)
/*****************************************************/
/* -- Test if the given key-value pair exists in the */
/*    given ini file.                                */
/* -- Return 1 if so, else 0.                        */
/* -- Return -1 in case of error (file does not      */
/*    exist, out of memory, etc).                    */
/* -- lszFile    : Full pathname of .ini file.       */
/* -- lszSection : Name of section in .ini file.     */
/* -- lszKey     : Key to look for in section.       */
/* -- lszValue   : Value of key to look for.         */
/*****************************************************/
    {
    int     nfd             = -1;
    int     wVal            = -1;
    int     cchKey          = lstrlen(lszKey);
    int     cchSection      = lstrlen(lszSection);
    int     cchValue        = lstrlen(lszValue);
    int     cch;
    char    szKeyValue[cbKeyValueMax];
    LPSTR   lpch;
    BOOL    fGotSection;

    /* Make sure key-value pair isn't too big. */
    if (cchKey + cchValue + 2 >= sizeof szKeyValue)
        goto FExistsSysIniKeyValueExit;

    /* Search for key-value pair in file. */
    if ((nfd = _lopen(lszFile, READ)) == -1)
        goto FExistsSysIniKeyValueExit;

    fGotSection = FALSE;
    for (;;)
        {
        int chSav;

        switch (WReadLineLsz(nfd, szKeyValue,
          sizeof szKeyValue))
            {
        default:
            goto FExistsSysIniKeyValueExit;

        case FALSE:
            wVal = 0;   /* Key not found. */
            goto FExistsSysIniKeyValueExit;

        case TRUE:
            break;
            }

        if (szKeyValue[0] == '[')   /* New section? */
            {
            if (fGotSection)
                {
                wVal = 0;
                goto FExistsSysIniKeyValueExit;
                }

            if (szKeyValue[cchSection + 1] == ']')
                {
                szKeyValue[cchSection + 1] = '\0';
                if (!lstrcmpi(szKeyValue + 1,
                  lszSection))
                    fGotSection = TRUE;
                }
            }
        else if (!fGotSection)
            continue;

        chSav = szKeyValue[cchKey];
        szKeyValue[cchKey] = '\0';
        if (lstrcmpi(szKeyValue, lszKey))
            continue;

        /* Skip white space. */
        szKeyValue[cchKey] = chSav;
        lpch = LpchSkipWhite(szKeyValue + cchKey);

        /* Must be an equal sign. */
        if (*lpch != '=')
            continue;

        /* Skip white space. */
        lpch = LpchSkipWhite(lpch + 1);

        /* Strip trailing white space. */
        if ((cch = lstrlen(lpch)) < cchValue)
            continue;

        for (;;)
            if (!FIsWhite(lpch[--cch]))
                break;
        lpch[++cch] = '\0';

        if (!lstrcmpi(lszValue, lpch))
            {
            wVal = 1;
            goto FExistsSysIniKeyValueExit;
            }
        }

FExistsSysIniKeyValueExit:
    if (nfd != -1)
        _lclose(nfd);
    return wVal;
    }

BOOL
FIsWhite(int ch)
/*****************************************************/
/* -- Return TRUE if the given character is white    */
/*    space.                                         */
/* -- ch : Character to test.                        */
/*****************************************************/
    {
    return ch == ' ' || ch == '\t' || ch == '\n' ||
      ch == '\f' || ch == '\r';
    }

LPSTR
LpchSkipWhite(LPSTR lpch)
/*****************************************************/
/* -- Skip leading white space.                      */
/* -- lpch : String to test.                         */
/*****************************************************/
    {
    while (*lpch)
        if (!FIsWhite(*lpch++))
            break;

    return --lpch;
    }

int
WReadLineLsz(int nfd, LPSTR lpch, int cb)
/*****************************************************/
/* -- Read a line from the given file.               */
/* -- Return false is no more to read.               */
/* -- Return -1 if current line exceeds buffer size. */
/* -- nfd  : File descriptor.                        */
/* -- lpch : Buffer to read into.                    */
/* -- cb   : Length of buffer.                       */
/*****************************************************/
    {
    for (;;)
        {
        if (!cb--)
            break;

        if (_lread(nfd, lpch, 1) < 1)
            return FALSE;

        if (*lpch++ == '\n')
            {
            *--lpch = '\0';
            return TRUE;
            }
        }

    return -1;
    }
