/*****************************************************/
/* systimer.c                                        */
/* -- Test application and system timer allocation.  */
/*****************************************************/

#include <windows.h>
#include "systimer.h"

/* Private. */
LONG    CALLBACK    MainWndProc(HWND, UINT, WPARAM,
                                LPARAM);
VOID                PaintWnd(HWND);

/* Imports. */
BOOL    WINAPI KillSystemTimer(HWND, UINT);
UINT    WINAPI SetSystemTimer(HWND, UINT, UINT,
                              TIMERPROC);

HINSTANCE   hins;       /* Instance of app. */
int         ctmr;       /* # timers allocated. */
int         ctmrSys;    /* # sys timers allocated. */

/* Class name. */
char        szApp[]    = "SysTimer";

int PASCAL
WinMain(HINSTANCE hinsThis, HINSTANCE hinsPrev,
  LPSTR lszCommand, int wShowWindow)
/*****************************************************/
/* -- Entry point.                                   */
/*****************************************************/
    {
    MSG     msg; 
    HWND    hwnd;

    hins = hinsThis;
    if (!hinsPrev) 
        {
        WNDCLASS    wcs; 

        wcs.style = CS_HREDRAW | CS_VREDRAW; 
        wcs.lpfnWndProc = MainWndProc;
        wcs.cbClsExtra = 0; 
        wcs.cbWndExtra = 0; 
        wcs.hInstance = hins; 
        wcs.hIcon = NULL;
        wcs.hCursor = LoadCursor(NULL, IDC_ARROW);
        wcs.hbrBackground = (HBRUSH)(COLOR_WINDOW + 1);
        wcs.lpszMenuName = szApp;
        wcs.lpszClassName = szApp;
        if (!RegisterClass(&wcs))
            return FALSE;
        }

    if (!(hwnd = CreateWindow(szApp, "Timer Test",
      WS_OVERLAPPEDWINDOW,
      CW_USEDEFAULT, CW_USEDEFAULT,
      GetSystemMetrics(SM_CXSCREEN) / 2,
      GetSystemMetrics(SM_CYSCREEN) / 4,
      NULL, NULL, hins, NULL)))
        return FALSE;

    ShowWindow(hwnd, wShowWindow);
    UpdateWindow(hwnd);

    while (GetMessage(&msg, NULL, NULL, NULL)) 
        {
        TranslateMessage(&msg);
        DispatchMessage(&msg);
        }

    return msg.wParam;
    }

LRESULT CALLBACK
MainWndProc(HWND hwnd, UINT wm, WPARAM wParam,
  LPARAM lParam)
/*****************************************************/
/* -- Main window procedure.                         */
/*****************************************************/
    {
    switch (wm)
        {
    default:
        return DefWindowProc(hwnd, wm, wParam, lParam);

    case WM_COMMAND:
        switch (wParam)
            {
        default:
            return
              DefWindowProc(hwnd, wm, wParam, lParam);

        case idmGet:
            for (ctmr = 0;
              SetTimer(hwnd, ctmr + 1, 1000, NULL);
              ctmr++)
                ;
            break;

        case idmFree:
            for (; ctmr > 0; ctmr--)
                KillTimer(hwnd, ctmr + 1);
            break;

        case idmSysGet:
            for (ctmrSys = 0;
              SetSystemTimer(hwnd, ctmrSys + 1, 1000,
                NULL); ctmrSys++)
                ;
            break;

        case idmSysFree:
            for (; ctmrSys > 0; ctmrSys--)
                KillSystemTimer(hwnd, ctmrSys + 1);
            break;
            }

        InvalidateRect(hwnd, NULL, TRUE);
        break;

    case WM_DESTROY:
        PostQuitMessage(0);
        break;

    case WM_PAINT:
        PaintWnd(hwnd);
        break;
        }  

    return 0;
    }

VOID
PaintWnd(HWND hwnd)
/*****************************************************/
/* -- Paint the main window's client area.           */
/* -- hwnd  : Window to paint in.                    */
/*****************************************************/
    {
    PAINTSTRUCT wps;
    char        szBuf[100];

    BeginPaint(hwnd, &wps);
    wsprintf(szBuf, "Timers allocated       : %d",
      ctmr);
    TextOut(wps.hdc, 0, 0, szBuf, lstrlen(szBuf));
    wsprintf(szBuf, "System Timers allocated: %d",
      ctmrSys);
    TextOut(wps.hdc, 0, HIWORD(GetDialogBaseUnits()),
      szBuf, lstrlen(szBuf));
    EndPaint(hwnd, &wps);
    }
